@@ -9,16 +9,16 @@
  * except according to those terms.
  */
 
-use std::borrow::Cow;
-
 use crate::parsers::message::MessageStream;
 
+use super::DecodeResult;
+
 pub fn decode_base64<'x>(
     stream: &MessageStream<'x>,
     start_pos: usize,
     boundary: &[u8],
     is_word: bool,
-) -> (usize, Option<Cow<'x, [u8]>>) {
+) -> (usize, DecodeResult) {
     let mut success = boundary.is_empty();
 
     let mut chunk: u32 = 0;
@@ -95,16 +95,19 @@ pub fn decode_base64<'x>(
         if success { bytes_read } else { 0 },
         if !buf.is_empty() {
             buf.shrink_to_fit();
-            Some(buf.into())
+            DecodeResult::Owned(buf)
         } else {
-            None
+            DecodeResult::Empty
         },
     )
 }
 
 #[cfg(test)]
 mod tests {
-    use crate::{decoders::base64::decode_base64, parsers::message::MessageStream};
+    use crate::{
+        decoders::{base64::decode_base64, DecodeResult},
+        parsers::message::MessageStream,
+    };
 
     #[test]
     fn decode_base64_strings() {
@@ -172,7 +175,11 @@ mod tests {
             );
 
             if !input.1.is_empty() {
-                let result_str = std::str::from_utf8(result.as_ref().unwrap().as_ref()).unwrap();
+                let bytes = match result {
+                    DecodeResult::Owned(v) => v,
+                    _ => unreachable!(),
+                };
+                let result_str = std::str::from_utf8(&bytes).unwrap();
                 //println!("'{}' -> '{}'", input.0.escape_debug(), result_str.escape_debug());
                 assert_eq!(
                     result_str,
